@@ -31,22 +31,25 @@ import statvfs
 from gettext import gettext as _
 
 # Private modules
-import operations
-import urlutil
+from serpentine import operations
+from serpentine import urlutil
 
-from mastering import GtkMusicList, MusicListGateway
-from mastering import ErrorTrapper
-from recording import ConvertAndWrite
-from preferences import RecordingPreferences
-from components import Component
-from mainwindow import SerpentineWindow
-from common import SerpentineNotSupportedError, SerpentineCacheError
-from plugins import plugins
+from serpentine.operations import (CallbackListener, SuccessfulOperation,
+                                   OperationsQueue, Listenable,
+                                   CallableOperation)
+from serpentine.mastering import GtkMusicList, MusicListGateway
+from serpentine.mastering import ErrorTrapper
+from serpentine.recording import ConvertAndWrite
+from serpentine.preferences import RecordingPreferences
+from serpentine.components import Component
+from serpentine.mainwindow import SerpentineWindow
+from serpentine.common import SerpentineNotSupportedError, SerpentineCacheError
+from serpentine.plugins import plugins
 
-PATTERN_SEPARATOR=";"
+PATTERN_SEPARATOR = ";"
 
 def _validate_music_list (music_list, preferences):
-    # Check if we have space available in our cache dir
+    """Check if we have space available in our cache dir"""
     secs = 0
     missing_musics = []
     for music in music_list:
@@ -68,97 +71,125 @@ def _validate_music_list (music_list, preferences):
     size_needed = secs * 176400L
     
     # Now check if cache location is ok
-    for tmp in preferences.get_temporary_dirs():
+    for tmp in preferences.temporary_dirs:
         try:
-            s = os.statvfs(tmp)
+            stat = os.statvfs(tmp)
         except OSError:
             # skip it if there's an error
             continue
         
         # Now check for available size
-        size_avail = s[statvfs.F_BAVAIL] * long(s[statvfs.F_BSIZE])
+        size_avail = stat[statvfs.F_BAVAIL] * long(stat[statvfs.F_BSIZE])
         if (size_avail - size_needed) < 0:
             preferences.pool.clear ()
         
-        size_avail = s[statvfs.F_BAVAIL] * long(s[statvfs.F_BSIZE])
+        size_avail = stat[statvfs.F_BAVAIL] * long(stat[statvfs.F_BSIZE])
         
         if size_avail - size_needed >= 0:
-            # Yay we have the needed space, update the preferences entry
-            preferences.temporary_dir = tmp
+            # Yay we have the needed space
             return
             
-    raise SerpentineCacheError (SerpentineCacheError.INVALID, _("Please "
+    raise SerpentineCacheError(SerpentineCacheError.INVALID, _("Please "
                             "check if the cache location exists and "
                             "has writable permissions."))
 
-class SavePlaylistRegistry (Component):
-    def __init__ (self, parent):
-        super (SavePlaylistRegistry, self).__init__ (parent)
+
+
+class SavePlaylistRegistry(Component, Listenable):
+    """Component responsible for keeping a registry of objects
+    that can save playlists."""
+    def __init__(self, parent):
+        Component.__init__(self, parent)
+        Listenable.__init__(self)
         
         # all files filter
-        all_files = gtk.FileFilter ()
-        all_files.set_name (_("All files"))
-        all_files.add_pattern ("*.*")
+        all_files = gtk.FileFilter()
+        all_files.set_name(_("All files"))
+        all_files.add_pattern("*")
         
         self.__global_filter = gtk.FileFilter()
-        self.__global_filter.set_name (_("Supported playlists"))
+        self.__global_filter.set_name(_("Supported playlists"))
         self.__file_filters = [self.__global_filter, all_files]
         self.__factories = {}
-        self.listeners = []
         
-    file_filters = property (lambda self: self.__file_filters)
+    file_filters = property(lambda self: self.__file_filters)
     
     def register (self, factory, extension, description):
         self.__global_filter.add_pattern ("*" + extension)
         
-        filter = gtk.FileFilter ()
-        filter.set_name (description)
-        filter.add_pattern ("*" + extension)
+        file_filter = gtk.FileFilter ()
+        file_filter.set_name (description)
+        file_filter.add_pattern ("*" + extension)
         
-        self.file_filters.append (filter)
-        
-        for listener in self.listeners:
-            listener.on_registred (factory, extension, description)
+        self.file_filters.append (file_filter)
+
+        self._notify('on_registred', factory, extension, description)
         
         self.__factories[extension] = factory
         self.__global_filter.add_pattern ("*" + extension)
     
-    def save (self, filename, extension = None):
+    def save(self, filename, extension=None):
         if extension is None:
-            fname, extension = os.path.splitext (filename)
+            fname, extension = os.path.splitext(filename)
 
         try:
-            return  self.__factories[extension](self.parent.music_list, filename)
+            return  self.__factories[extension](self.parent.music_list,
+                                                filename)
         except KeyError:
             raise SerpentineNotSupportedError (extension)
     
-class Application (operations.Operation, Component):
+class LeaseMusicList(object):
+    def __init__(self, music_list, pool):
+        self.music_list = music_list
+        self.pool = pool
+        
+    def request_music_list(self):
+        self.requested = list(row['location']
+                              for row in self.music_list)
+        for location in self.requested:
+            self.pool.request_music(location)
+
+    def release_music_list(self):
+        for location in self.requested:
+            self.pool.release_music(location)
+            
+
+class Application(operations.Operation, Component):
     components = ()
     def __init__ (self, locations):
-        operations.Operation.__init__ (self)
-        Component.__init__ (self, None)
-        self.savePlaylist = SavePlaylistRegistry (self)
+        operations.Operation.__init__(self)
+        Component.__init__(self, None)
+        self.savePlaylist = SavePlaylistRegistry(self)
         self.locations = locations
-        self.__preferences = RecordingPreferences (locations)
+        self.__preferences = RecordingPreferences(locations)
         self.__running_ops = []
         
         self._music_file_patterns = {}
         self._playlist_file_patterns = {}
         self._music_file_filters = None
         self._playlist_file_filters = None
-        self.register_music_file_pattern("MPEG Audio Stream, Layer III", "*.mp3")
-        self.register_music_file_pattern("Ogg Vorbis Codec Compressed WAV File", "*.ogg")
+        self.register_music_file_pattern("MPEG Audio Stream, Layer III",
+                                         "*.mp3")
+        self.register_music_file_pattern("Ogg Vorbis Codec Compressed WAV File",
+                                         "*.ogg")
         self.register_music_file_pattern("Free Lossless Audio Codec", "*.flac")
         self.register_music_file_pattern("MPEG 4 Audio", "*.m4a;*.mp4")
         self.register_music_file_pattern("PCM Wave audio", "*.wav")
+        self._writing_started = CallbackListener(self._on_writing_started,
+                                                 ('on_finished',))
+        self._writing_finished = CallbackListener(self._on_writing_finished,
+                                                  ('on_finished',))
     
     def _load_plugins (self):
         # Load Plugins
         self.__plugins = []
         for plug in plugins:
-            # TODO: Do not add plugins that throw exceptions calling this method
-            self.__plugins.append (plugins[plug].create_plugin (self))
-        
+            try:
+                self.__plugins.append(plugins[plug].create_plugin(self))
+            except Exception, e:
+                print "Error loading plugin", plug
+                import traceback
+                traceback.print_exc()
 
     cache_pool = property(lambda self: self.__preferences.pool)
 
@@ -171,19 +202,29 @@ class Application (operations.Operation, Component):
     # The window is only none when the operation has finished
     can_start = property(lambda self: self.__window is not None)
 
+    def get_music_list(self):
+        raise NotImplementedError()
+
+    music_list = property(get_music_list)
+
+    def get_music_list_gw(self):
+        raise NotImplementedError()
+
+    music_list_gw = property(get_music_list_gw)
 
     def on_finished (self, event):
-        # We listen to operations we contain, when they are finished we remove them
-        self.__running_ops.remove (event.source)
+        # We listen to operations we contain, when they are finished
+        # we remove them
+        self.__running_ops.remove(event.source)
         if self.can_stop:
-            self.stop ()
+            self.stop()
 
-    def stop (self):
+    def stop(self):
         assert self.can_stop, "Check if you can stop the operation first."
-        self.preferences.savePlaylist (self.music_list)
+        self.preferences.savePlaylist(self.music_list)
         self.preferences.pool.clear()
         # Warn listeners
-        self._send_finished_event (operations.SUCCESSFUL)
+        self._send_finished_event(operations.SUCCESSFUL)
             
         # Cleanup plugins
         del self.__plugins
@@ -195,18 +236,44 @@ class Application (operations.Operation, Component):
         _validate_music_list(music_list, self.preferences)
         self.validated = True
 
-    def write_files (self, window=None):
+    def write_files(self, window=None):
         # TODO: we should add a confirmation dialog
         if not self.validated:
             raise ValueError("Call validate_files first")
-            
-        r = ConvertAndWrite (self.music_list, self.preferences, window)
-        # Add this operation to the recording
-        self.running_ops.append (r)
-        r.listeners.append (self)
+
+        start = SuccessfulOperation()
+        start.add_listener(self._writing_started)
+
+        lease = LeaseMusicList(self.music_list, self.cache_pool)
+
+        request = CallableOperation(lease.request_music_list)
+
+        release = CallableOperation(lease.release_music_list)
+        
+        conv_write = ConvertAndWrite(self.music_list, self.preferences, window)
+
+        queue = OperationsQueue([start, request, conv_write, release])
+        queue.abort_on_failure = False
+
+        # remove this operation from the running ops
+        self.running_ops.append(queue)
+        queue.add_listener(self)
+        
+        queue.add_listener(self._writing_finished)
+        
         self.validated = False
-        return r
+        
+        return queue
+    
     write_files = operations.async(write_files)
+
+    def _on_writing_started(self, *args):
+        """Called by '_writing_started'"""
+        self._notify('on_writing_started', self)
+        
+    def _on_writing_finished(self, *args):
+        """Called by '_writing_finished'"""
+        self._notify('on_writing_finished', self)
     
     # TODO: should these be moved to MusicList object?
     # TODO: have a better definition of a MusicList
@@ -237,7 +304,7 @@ class Application (operations.Operation, Component):
     def __gen_file_filters(self, patterns, all_filters_name):
         all_files = gtk.FileFilter()
         all_files.set_name(_("All files"))
-        all_files.add_pattern("*.*")
+        all_files.add_pattern("*")
         
         all_musics = gtk.FileFilter()
         all_musics.set_name(all_filters_name)
@@ -302,6 +369,7 @@ class Application (operations.Operation, Component):
     def clear_files(self):
         self.music_list.clear()
 
+
 class HeadlessApplication (Application):
 
     
@@ -314,8 +382,8 @@ class HeadlessApplication (Application):
             def prepare_queue (self, gateway, queue):
                 self.trapper = ErrorTrapper (None)
             
-            def prepare_add_file (self, gateway, add_file):
-                add_file.listeners.append (self.trapper)
+            def prepare_add_file(self, gateway, add_file):
+                add_file.add_listener(self.trapper)
                 
             def finish_queue (self, gateway, queue):
                 queue.append (self.trapper)
@@ -327,10 +395,15 @@ class HeadlessApplication (Application):
         self.music_list_gw = HeadlessApplication.Gateway(self)
         self._load_plugins ()
 
-    music_list = property (lambda self: self.music_list_gw.music_list)
-    
+    def get_music_list(self):
+        return self.music_list_gw.music_list
 
-class SerpentineApplication (Application):
+    music_list = property(get_music_list)
+
+    music_list_gw = None
+
+
+class SerpentineApplication(Application):
     """When no operations are left in SerpentineApplication it will exit.
     An operation can be writing the files to cd or showing the main window.
     This enables us to close the main window and continue showing the progress
@@ -340,34 +413,51 @@ class SerpentineApplication (Application):
         Application.__init__(self, locations)
         self.__window = SerpentineWindow(self)
         self.preferences.dialog.set_transient_for(self.window_widget)
-        self.__window.listeners.append(self)
+        self.__window.add_listener(self)
         self._load_plugins()
 
 
     window_widget = property (lambda self: self.__window)
-    
-    music_list = property (lambda self: self.window_widget.music_list)
 
-    music_list_gw = property (lambda self: self.window_widget.masterer.music_list_gateway)
+    def get_music_list(self):
+        return self.window_widget.music_list
+
+    music_list = property(get_music_list)
+
+    def get_music_list_gw(self):
+        return self.window_widget.masterer.music_list_gateway
+
+    music_list_gw = property(get_music_list_gw)
     
-    def write_files (self):
-        return Application.write_files (self, self.window_widget)
+    def write_files(self, window=None):
+        """Writes the files onto the Audio-CD."""
+        super_self = super(SerpentineApplication, self)
+        return super_self.write_files(self.window_widget)
     
     # TODO: decouple the window from SerpentineApplication ?
-    def show_window (self):
+    def show_window(self):
+        """
+        Opens the main window.
+        """
         # Starts the window operation
         self.__window.start ()
         self.running_ops.append (self.__window)
     
-    def close_window (self):
+    def close_window(self):
+        """
+        Closes the main window.
+        """
         # Stops the window operation
         if self.__window.running:
             self.__window.stop ()
     
-    def stop (self):
+    def stop(self):
+        """
+        Stops the application.
+        """
         # Clean window object
-        Application.stop (self)
-        self.__window.destroy ()
+        Application.stop(self)
+        self.__window.destroy()
         del self.__window
         
 
