@@ -32,7 +32,10 @@ if __name__ == '__main__':
 import threading
 import gst
 import gobject
-import operations
+from serpentine import operations
+
+import logging
+logger = logging.getLogger('serpentine')
 
 _VOID_VOID = (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ())
 
@@ -53,7 +56,7 @@ if gst.gst_version < (0, 9):
 else:
     safe_element_factory_make = gst.element_factory_make
     
-class GstPlayingFailledError(StandardError):
+class GstPlayingFailedError(StandardError):
     """This error is thrown when we can't set the state to PLAYING"""
     
 class GstOperationListener(operations.OperationListener):
@@ -89,7 +92,7 @@ class GstPipelineOperation(operations.MeasurableOperation):
         self.__can_start = False
         self.__running = True
         if not self.bin.set_state(gst.STATE_PLAYING):
-            self._finalize(operations.ERROR, GstPlayingFailledError())
+            self._finalize(operations.ERROR, GstPlayingFailedError())
 
     def stop(self):
         self._finalize(operations.ABORTED)
@@ -122,8 +125,17 @@ class GstPipelineOperation(operations.MeasurableOperation):
                 position = self.query_position()
                 progress = self.query_position() / self.__duration
 
+            # Fix invalid values that migh occour
+            if progress < 0 or progress > 1:
+                logger.info("Progress with invalid value %r" % progress)
+                
+                if progress < 0:
+                    progress = 0
+                elif progress > 1:
+                    progress = 1
+
             self.__progress = max(self.__progress, progress)
-            assert 0 <= self.__progress <= 1, self.__progress
+                
             
         return self.__progress
 
@@ -320,7 +332,7 @@ class AudioMetadata(operations.Operation, GstOperationListener): #, gobject.GObj
         self._fakesink.set_property("signal-handoffs", True)
         self._fakesink.connect("handoff", self.on_handoff)
         self.__oper.query_element = self._fakesink
-        self.__oper.listeners.append(self) 
+        self.__oper.add_listener(self) 
         self.__metadata = {}
         self.__element = None
 
@@ -347,7 +359,7 @@ class AudioMetadata(operations.Operation, GstOperationListener): #, gobject.GObj
     def on_finished(self, event):
         self.duration_counter = 0
         
-        if event.id == operations.ERROR:
+        if event.is_error:
             self._propagate(event)
             return
             
@@ -448,7 +460,7 @@ class IsWavPcm(operations.Operation, GstOperationListener):
         )
         
         self.oper = GstOperation(pipeline = bin)
-        self.oper.listeners.append(self)
+        self.oper.add_listener(self)
  
         decoder = bin.get_by_name("iwp_typefind")
         
@@ -475,17 +487,15 @@ class IsWavPcm(operations.Operation, GstOperationListener):
     
     def on_finished(self, event):
         
-        if event.id != operations.ERROR and self.is_wav_pcm:
-            assert event.id == operations.SUCCESSFUL or \
-                   event.id == operations.ABORTED
-            
+        if not event.is_error and self.is_wav_pcm:
+            assert event.is_successful or event.is_aborted
             self._send_finished_event(operations.SUCCESSFUL)
         else:
-            if event.id == operations.SUCCESSFUL:
+            if event.is_successful:
                 eid = operations.ERROR
                 err = StandardError("Not a valid WAV PCM")
             else:
-                eid = event.id
+                eid = event.state
                 err = event.error
                 
             self._send_finished_event(eid, err)
@@ -569,29 +579,30 @@ if __name__ == '__main__':
             self.oper = oper
             
         def on_metadata(self, event, metadata):
-            print >> sys.stderr, metadata
+            #print >> sys.stderr, metadata
+	    pass
             
         def on_finished(self, event):
-            self.success = operations.SUCCESSFUL == event.id
-            if event.id == operations.ERROR:
+            self.success = event.is_successful
+            if event.is_error:
                 print "ERROR:", event.error
             mainloop.quit()
 
         def on_progress(self):
-            print self.oper.progress
+            #print self.oper.progress
             return True
     
     #f = convert_to_wav(FILE_SRC, sys.argv[1], sys.argv[2])
     f = parse_command(sys.argv[1], FILE_SRC, sys.argv[2], *sys.argv[3:])
     #f = is_wav_pcm(FILE_SRC, sys.argv[1])
-    #print operations.syncOperation(f).id == operations.SUCCESSFUL
+    #print operations.syncOperation(f).is_successful
     #raise SystemError
     #f = get_metadata("filesrc", sys.argv[1])
     #f = extractAudioTrackFile("/dev/cdrom", int(sys.argv[1]) + 1, sys.argv[2])
     l = Listener(f)
     if isinstance(f, operations.MeasurableOperation):
         gobject.timeout_add(200, l.on_progress)
-    f.listeners.append(l)
+    f.add_listener(l)
     f.start()
     l.finished = False
 
