@@ -97,7 +97,7 @@ def __plural (value, strings):
         return strings[1]
 
 
-class SafeFileWrite:
+class SafeFileWrite(object):
     """This class enables the user to safely write the contents to a file and
     if something wrong happens the original file will not be damaged. It writes
     the contents in a temporary file and when the file descriptor is closed the
@@ -145,3 +145,11 @@ def parse_key_event(evt):
     args.append(gdk.keyval_name(evt.keyval))
     return "+".join(args)
     
+
+GCONF_DIR = "/apps/serpentine"
+
+RAT_GCONF_DIR = "/apps/rat"
+
+NCB_GCONF_DIR = "/apps/nautilus-cd-burner"
+
+KEY_TEMP_ISO_DIR = NCB_GCONF_DIR + "/temp_iso_dir"
