@@ -44,7 +44,7 @@ class DictModelIterator:
         self.index += 1
         return ret
         
-class DictStore (gtk.ListStore):
+class DictStore(gtk.ListStore):
     def __init__ (self, *cols):
         self.indexes = {}
         self.cols = []
@@ -75,13 +75,23 @@ class DictStore (gtk.ListStore):
         
     def index_of (self, key):
         return self.indexes[key]
+    
     def get (self, index):
         return DictModelRow(self, self[index])
+
     def __iter__ (self):
         return DictModelIterator(self)
-        
+
+    def __repr__(self):
+        return repr(tuple(iter(self)))
+
+
+
+
 gobject.type_register (DictStore)
 
+
+
 class DictModelRow (object):
     def __init__ (self, parent, row):
         self.parent = parent
@@ -109,6 +119,11 @@ class DictModelRow (object):
 
     __contains__ = has_key
 
+
+    def __repr__(self):
+        return repr(tuple(self.row))
+
+
 class SimpleListWrapper:
     def __init__ (self, store):
         self.store = store
@@ -119,6 +134,7 @@ class SimpleListWrapper:
     def append (self, item):
         self.store.append ([item])
 
+
 class SimpleList (gtk.TreeView):
     def __init__ (self, column_title, editable = True):
         gtk.TreeView.__init__(self)
@@ -142,6 +158,8 @@ class SimpleList (gtk.TreeView):
     def __on_text_edited (self, cell, path, new_text, user_data = None):
         self.get_model()[path][0] = new_text
 
+    def __repr__(self):
+        return repr([tuple(r) for r in self.get])
 gobject.type_register (SimpleList)
 
 ################################################################################
@@ -664,7 +682,8 @@ class _SingleListStrategy(_BaseStrategy):
         rows = list_view.get_selection().get_selected_rows()[1]
         get_element = lambda row: row[0]
 
-        items = tuple(map(get_element, rows))
+        return tuple(map(get_element, rows))
+        
         
 _STRATEGIES = (_OneStrategy, _MultipleStrategy, _RadioStrategy,
                _SingleListStrategy)
@@ -740,7 +759,7 @@ def choice_dialog(primary_text, secondary_text, parent=None, \
     if response != gtk.RESPONSE_OK:
         dlg.destroy()
         return (), response
-    
+
     items = strategy.get_items(dlg)
     dlg.destroy()
     
@@ -1160,4 +1179,4 @@ class HigProgress(gtk.Window):
         # click on the cancel button
         self.cancel_button.clicked()
         # let the clicked event close the window if it likes too
-        return True
\ No newline at end of file
+        return True
