@@ -21,20 +21,17 @@ import os
 import os.path
 import gobject
 import webbrowser
-
-# Local imports
-import operations
-import gaw
-import gtkutil
 import gconf
 
-from components import Component
-from operations import MapProxy, OperationListener
-from mastering import AudioMastering
-from common import parse_key_event
-
-from serpentine.common import SerpentineNotSupportedError
-from serpentine.common import SerpentineCacheError
+# Local imports
+from serpentine import (operations, gaw, gtkutil)
+from serpentine.preferences import KEY_TEMP_ISO_DIR
+from serpentine.components import Component
+from serpentine.operations import MapProxy, OperationListener
+from serpentine.mastering import AudioMastering
+from serpentine.gtkutil import SetupAlert
+from serpentine.common import (SerpentineNotSupportedError, parse_key_event,
+                               SerpentineCacheError)
 from gettext import gettext as _
 
 D_G_INTERFACE = "/desktop/gnome/interface"
@@ -46,7 +43,7 @@ def open_url(dlg, url):
 gtk.about_dialog_set_url_hook(open_url)
 
 for gconf_dir in (D_G_INTERFACE,):
-    gconf.client_get_default ().add_dir (gconf_dir, gconf.CLIENT_PRELOAD_NONE)
+    gconf.client_get_default().add_dir(gconf_dir, gconf.CLIENT_PRELOAD_NONE)
 
 class GladeComponent (Component):
 
@@ -109,6 +106,7 @@ class FileDialogComponent (GladeComponent):
         for filter in file_filters:
             self.file_dlg.add_filter (filter)
 
+
 class AddFileComponent (FileDialogComponent):
     def _setup_glade (self, g):
         g.get_widget ("add").connect ("clicked", self.run_dialog)
@@ -123,7 +121,7 @@ class AddFileComponent (FileDialogComponent):
     _get_file_filters = lambda self: self.parent.application.music_file_filters
 
     
-class PlaylistComponent (FileDialogComponent):
+class PlaylistComponent(FileDialogComponent):
     component_name = "open_playlist"
     def _setup_glade (self, g):
         g.get_widget ("open_playlist_mni").connect ("activate", self.run_dialog)
@@ -136,7 +134,8 @@ class PlaylistComponent (FileDialogComponent):
         self.parent.music_list_widget.music_list_gateway.add_files ([playlist]).start ()
         self.parent.clear_files ()
 
-class SavePlaylistComponent (GladeComponent):
+
+class SavePlaylistComponent(GladeComponent):
     component_name = "save_playlist"
     
     def _setup_glade (self, g):
@@ -173,7 +172,7 @@ class SavePlaylistComponent (GladeComponent):
         
         self.file_dlg.set_extra_widget (hbox)
         app = self.parent.application
-        app.savePlaylist.listeners.append (self)
+        app.savePlaylist.add_listener(self)
         self.__sync = True
     
     def on_registred (self, factory, extension, description):
@@ -183,7 +182,7 @@ class SavePlaylistComponent (GladeComponent):
     def on_finished (self, evt):
         win = self.parent
             
-        if evt.id != operations.SUCCESSFUL:
+        if not evt.is_successful:
             gtkutil.dialog_error (
                 _("Playlist Not Saved"),
                 _("There was an error while saving the playlist."),
@@ -236,7 +235,7 @@ class SavePlaylistComponent (GladeComponent):
             
             try:
                 oper = app.savePlaylist.save (filename, extension)
-                oper.listeners.append (self)
+                oper.add_listener(self)
                 oper.start ()
                 break
 
@@ -273,7 +272,7 @@ class SavePlaylistComponent (GladeComponent):
                         
                         # Save the file
                         oper = app.savePlaylist.save (filename, extension)
-                        oper.listeners.append (self)
+                        oper.add_listener(self)
                         oper.start ()
                         
                         # Select the option in the list store
@@ -293,6 +292,7 @@ class SavePlaylistComponent (GladeComponent):
         self.file_dlg.unselect_all()
         self.file_dlg.hide()
 
+
 class ToolbarComponent (GladeComponent):
     Style = {
         "both": gtk.TOOLBAR_BOTH,
@@ -384,6 +384,7 @@ class ToolbarComponent (GladeComponent):
             
     style = property (style)
 
+
 class PersistencePreferences(GladeComponent):
     prefs = property(lambda self: self.parent.preferences)
     
@@ -433,6 +434,7 @@ class PersistencePreferences(GladeComponent):
         except ValueError:
             pass
 
+
 class KeyBindings(GladeComponent):
     
     def init(self):
@@ -464,7 +466,34 @@ class KeyBindings(GladeComponent):
     def on_save_playlist(self):
         self.parent.save_playlist()
     
-class SerpentineWindow (gtk.Window, OperationListener, operations.Operation, Component):
+
+class SetupCacheAlert(SetupAlert):
+    def _after_text(self, dialog, vbox):
+        hbox = gtk.HBox(spacing=6)
+        
+        hbox.pack_start(gtk.Label(_('Cache location:')), expand=False,
+                        fill=False)
+        
+        btn = gtk.FileChooserButton('Select a File')
+        btn.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
+        client = gconf.client_get_default()
+        data = client.get(KEY_TEMP_ISO_DIR)
+        btn.set_current_folder(data.to_string())
+        def on_changed(filechooser):
+            client.set_string(KEY_TEMP_ISO_DIR,
+                              filechooser.get_filename())
+            
+        btn.connect('selection-changed', on_changed)
+        
+        hbox.add(btn)
+        
+        hbox.show_all()
+
+        vbox.pack_start(hbox)
+        
+
+class SerpentineWindow(gtk.Window, OperationListener, operations.Operation,
+                       Component):
     # TODO: finish up implementing an Operation
     components = (
         AddFileComponent,
@@ -488,7 +517,7 @@ class SerpentineWindow (gtk.Window, OperationListener, operations.Operation, Com
         self.__running = False
         self.connect ("show", self.__on_show)
         # We listen for GtkMusicList events
-        self.music_list_widget.listeners.append(self)
+        self.music_list_widget.add_listener(self)
         
         glade_file = application.locations.get_data_file("serpentine.glade")
         g = gtk.glade.XML (glade_file, "main_window_container")
@@ -666,6 +695,7 @@ class SerpentineWindow (gtk.Window, OperationListener, operations.Operation, Com
             gtkutil.dialog_warn(
                 title, 
                 gobject.markup_escape_text(err.error_message),
+                _setup_alert=SetupCacheAlert,
                 parent=self
             )
             
@@ -688,7 +718,8 @@ class SerpentineWindow (gtk.Window, OperationListener, operations.Operation, Com
             btn = _("Write to Disc")
             self.__application.preferences.overburn = False
         
-        if gtkutil.dialog_ok_cancel (title, msg, parent = self, ok_button = btn) != gtk.RESPONSE_OK:
+        if gtkutil.dialog_ok_cancel(title, msg, parent=self,
+                                    ok_button=btn) != gtk.RESPONSE_OK:
             return
         
         self.application.write_files().start()
