@@ -21,6 +21,9 @@ import gtk
 import gobject
 import sys
 import weakref
+import logging
+
+logger = logging.getLogger('serpentine')
 
 from gtk import glade
 from os import path
@@ -28,17 +31,25 @@ from types import IntType, TupleType
 from gettext import gettext as _
 from gettext import ngettext as N_
 
-# Local modules
-import operations
-import audio
-import xspf
-import gtkutil
-import urlutil
+from serpentine.errors import UnsupportedLocationError
 
-from gtkutil import DictStore
-from operations import OperationsQueue
-from gdkpiechart import SerpentineUsage
-from common import parse_key_event
+# Local modules
+from serpentine import operations
+from serpentine import audio
+from serpentine import xspf
+from serpentine import gtkutil
+from serpentine import urlutil
+
+from serpentine.gtkutil import DictStore
+from serpentine.operations import OperationsQueue
+from serpentine.gdkpiechart import SerpentineUsage
+from serpentine.common import parse_key_event
+
+# Python 2.3 compability
+try:
+    set
+except NameError:
+    from sets import Set as set
 
 ################################################################################
 # Operations used on AudioMastering
@@ -55,14 +66,12 @@ class ErrorTrapper (operations.Operation, operations.OperationListener):
     parent = property (lambda self: self._parent)
     
     def on_finished (self, event):
-        if event.id == operations.ERROR:
+        if event.is_error:
             self.errors.append (event.source)
     
     def start (self):
         if len (self.errors) == 0:
-            e = operations.FinishedEvent (self, operations.SUCCESSFUL)
-            for l in self.listeners:
-                l.on_finished (e)
+            self._send_finished_event(operations.SUCCESSFUL)
             return
 
         filenames = []
@@ -93,11 +102,9 @@ class ErrorTrapper (operations.Operation, operations.OperationListener):
             buttons =(gtk.STOCK_CLOSE, gtk.RESPONSE_OK),
         )
         
-        e = operations.FinishedEvent (self, operations.SUCCESSFUL)
-        for l in self.listeners:
-            l.on_finished (e)
+        self._send_finished_event(operations.SUCCESSFUL)
 
-class AddFile (audio.AudioMetadataListener, operations.Operation):
+class AddFile(operations.Operation, audio.AudioMetadataListener):
     # TODO: Implement full Operation here
     
     running = property (lambda self: False)
@@ -118,7 +125,7 @@ class AddFile (audio.AudioMetadataListener, operations.Operation):
         else:
             url = urlutil.UrlParse(self.hints["location"])
             if not url.is_local:
-                self._send_finished_error(
+                self._send_finished_event(
                     operations.ERROR,
                     error=StandardError(self.hints["location"])
                 )
@@ -127,10 +134,10 @@ class AddFile (audio.AudioMetadataListener, operations.Operation):
             filename = url.path
             oper = audio.get_metadata("filesrc", filename)
 
-        oper.listeners.append (self)
+        oper.add_listener(self)
         try:
             oper.start()
-        except audio.GstPlayingFailledError:
+        except audio.GstPlayingFailedError:
             self._send_finished_event(
                 operations.ERROR,
                 error=StandardError(self.hints["location"])
@@ -142,7 +149,6 @@ class AddFile (audio.AudioMetadataListener, operations.Operation):
         
         row = {
             "location": self.hints['location'],
-            "cache_location": "",
             "title": title or _("Unknown"),
             "artist": _("Unknown Artist"),
             "duration": int(metadata['duration']),
@@ -164,10 +170,8 @@ class AddFile (audio.AudioMetadataListener, operations.Operation):
             self.music_list.append (row)
         
     
-    def on_finished (self, evt):
-        e = operations.FinishedEvent (self, evt.id)
-        for l in self.listeners:
-            l.on_finished (e)
+    def on_finished(self, evt):
+        self._propagate(evt)
             
  
 class UpdateDiscUsage (operations.Operation):
@@ -184,18 +188,17 @@ class UpdateDiscUsage (operations.Operation):
         self.__masterer.update = self.__update
         if self.__update:
             self.__masterer.update_disc_usage()
-        e = operations.FinishedEvent (self, operations.SUCCESSFUL)
-        for l in self.listeners:
-            l.on_finished (e)
+        self._send_finished_event(operations.SUCCESSFUL)
 
 ################################################################################
 
-class MusicListListener:
-    def on_musics_added (self, event, rows):
-        pass
+class MusicListListener(object):
+    """The interface from a music list listener."""
+    def on_musics_added (self, event):
+        """Musics were added to the source of the event."""
     
-    def on_musics_removed (self, event, rows):
-        pass
+    def on_musics_removed (self, event):
+        """Musics were removed from the source of the event."""
 
 class MusicList (operations.Listenable):
     def __getitem__ (self):
@@ -246,7 +249,8 @@ class MusicList (operations.Listenable):
             t.creator = r['artist']
             playlist.tracks.append (t)
 
-class GtkMusicList (MusicList):
+
+class GtkMusicList(MusicList):
     """The GtkMusicList uses a ListStore as a backend, it is not visual and
     depends only on glib.
     
@@ -255,8 +259,6 @@ class GtkMusicList (MusicList):
     SPEC = (
         # URI is used in converter
         {"name": "location", "type": gobject.TYPE_STRING},
-        # filename is used in recorder
-        {"name": "cache_location", "type": gobject.TYPE_STRING},
         # Remaining items are for the list
         {"name": "duration", "type": gobject.TYPE_INT},
         {"name": "title", "type": gobject.TYPE_STRING},
@@ -265,75 +267,100 @@ class GtkMusicList (MusicList):
     )
             
     def __init__ (self):
-        operations.Listenable.__init__ (self)
-        self.__model = DictStore (*self.SPEC)
+        operations.Listenable.__init__(self)
+        self.__model = DictStore(*self.SPEC)
+        self.__model.connect("row-deleted", self._on_row_deleted)
+        self.__model.connect("row-inserted", self._on_row_inserted)
         self.__total_duration = 0
-        self.__freezed = False
+        self.__frozen = False
+        self.use_gap = False
     
-    model = property (fget=lambda self: self.__model, doc="Associated ListStore.")
-    total_duration = property (fget=lambda self:self.__total_duration, doc="Total disc duration, in seconds.")
+    model = property (fget=lambda self: self.__model,
+                      doc="Associated ListStore.")
+
+    def _recalculate(self):
+        self.__total_duration = 0
+        for row in self:
+            self.__total_duration += row['duration']
+
+    def _notify_removed(self):
+        if not self.__frozen:
+            e = operations.Event (self)
+            gobject.idle_add(lambda: self._notify('on_musics_removed', e))
+
+    def _notify_added(self):
+        if not self.__frozen:
+            e = operations.Event (self)
+            gobject.idle_add(lambda: self._notify('on_musics_added', e))
+    
+    def _on_row_deleted(self, source, rows):
+        self._recalculate()
+        self._notify_removed()
+
+    def _on_row_inserted(self, source, path, tree_iter):
+        self._notify_added()
+    
+    def get_total_duration(self):
+        if len(self) == 0:
+            assert self.__total_duration == 0
+            return 0
+
+        total = self.__total_duration
+        if self.use_gap:
+            total += (len(self) - 1) * 2
+            
+        return total
+
+    total_duration = property(get_total_duration,
+                              doc="Total disc duration, in seconds.")
     
     def __getitem__ (self, index):
         return self.model.get (index)
     
+
     def append_many (self, rows):
-        self.__freezed = True
+        
+        self.__frozen = True
         for row in rows:
             self.append (row)
-        self.__freezed = False
-        
-        rows = tuple(rows)
-        e = operations.Event(self)
-        for l in self.listeners:
-            l.on_musics_added (e, rows)
+        self.__frozen = False
+
+        self._notify_added()
     
+
     def __correct_row (self, row):
         if not row.has_key ('time'):
             row['time'] = "%.2d:%.2d" % (row['duration'] / 60, row['duration'] % 60)
-        if not row.has_key ('cache_location'):
-            row['cache_location'] = ''
         return row
         
+
     def append (self, row):
         row = self.__correct_row(row)
-        self.model.append (row)
-        self.__total_duration += int(row['duration'])
-
-        if not self.__freezed:
-            e = operations.Event (self)
-            rows = (row,)
-            for l in self.listeners:
-                l.on_musics_added (e, rows)
+        self.model.append(row)
+        self.__total_duration += row['duration']
     
+
     def insert (self, index, row):
-
         row = self.__correct_row(row)
-        self.model.insert_before (self.model.get_iter (index), row)
-        self.__total_duration += int (row['duration'])
-
-        if not self.__freezed:
-            e = operations.Event (self)
-            rows = (row,)
-            for l in self.listeners:
-                l.on_musics_added (e, rows)
+        self.model.insert_before(self.model.get_iter (index), row)
+        self.__total_duration += row['duration']
         
+
     def __len__ (self):
         return len(self.model)
     
+
     def __delitem__ (self, index):
         # Copy native row
         row = dict(self[index])
         del self.model[index]
         self.__total_duration -= row['duration']
         rows = (row,)
-        if not self.__freezed:
-            e = operations.Event (self)
-            for l in self.listeners:
-                l.on_musics_removed (e, rows)
     
+
     def delete_many (self, indexes):
         assert isinstance(indexes, list)
-        rows = []
+
         indexes.sort()
         low = indexes[0] - 1
         # Remove duplicate entries
@@ -344,64 +371,109 @@ class GtkMusicList (MusicList):
         # Now decrement the offsets
         for i in range (len (indexes)):
             indexes[i] -= i
-        
+
         # Remove the elements directly
+        self.__frozen = True
         for i in indexes:
             # Copy native row
             r = dict(self.model.get(i))
-            rows.append(r)
             self.__total_duration -= r['duration']
             del self.model[i]
+        self.__frozen = False
         
         # Warn the listeners
-        rows = tuple(rows)
-        e = operations.Event(self)
-        for l in self.listeners:
-            l.on_musics_removed (e, rows)
+        self._notify_removed()
         
     def clear (self):
-        rows = []
-        for row in iter(self.model):
-            # Copy each element
-            rows.append(dict(row))
-        
+        self.__frozen = True
         self.model.clear()
+        self.__frozen = False
         self.__total_duration = 0
-        
-        rows = tuple(rows)
-        e = operations.Event (self)
-        for l in self.listeners:
-            l.on_musics_removed(e, rows)
-    
+
+        # Warn the listeners
+        self._notify_removed()
+
+    def __repr__(self):
+        return repr(self.model)
+
+
 
 ################################################################################
 # Audio Mastering widget
 #    
 
+def copy_store(store):
+    """
+    Convert a store to a dictionary of filenames and their usage.
+    """
+    result = {}
+    for row in store:
+        location = row['location']
+        try:
+            result[location] += 1
+        except:
+            result[location] = 1
+            
+    return result
+    
+
+def diff_stores(old_store, new_store):
+    """
+    Creates a new dictionary that represents which locations
+    were removed and which are added.
+    """
+    diff = {}
+    for location in new_store:
+        old_value = old_store.get(location, 0)
+        new_value = new_store.get(location, 0)
+        diff[location] = new_value - old_value
+
+    for location in old_store:
+        if location not in new_store:
+            diff[location] = -old_store[location]
+            
+    return diff
+
+
 class AudioMasteringMusicListener(MusicListListener):
     def __init__ (self, audio_mastering):
-        self.__master = audio_mastering
+        self.master = audio_mastering
+        self.old_store = {}
+
+    def update_pool(self):
+        """
+        Updates the music pool.
+        """
+        store = copy_store(self.master.source)
+
+        diff = diff_stores(old_store=self.old_store,
+                           new_store=store)
+
+        for (location, value) in diff.items():
+            if value > 0:
+                for x in range(value):
+                    self.master.pool.request_music(location)
+            else:
+                for x in range(-value):
+                    self.master.pool.release_music(location)
+
+        self.old_store = store
+
         
-    def on_musics_added (self, e, rows):
-        self.__master.update_disc_usage()
-    
-    def on_musics_removed (self, e, rows):
-        self.__master.update_disc_usage()
+    def on_musics_added(self, evt):
+        self.master.update_disc_usage()
+        self.update_pool()
         
-        for row in rows:
-            self.__master.pool.remove_music(row['location'])
+
+    def on_musics_removed(self, evt):
+        self.master.update_disc_usage()
+        self.update_pool()
+
             
 
-class HintsFilter (object):
-    __priority = 0
-    
-    def priority (self, value):
-        assert isinstance (value, int)
-        self.__priority = value
-        
-    priority = property (lambda self: self.__priority, priority, doc="Represents " \
-    "the parser priority, a higher value will give it precedence over " \
-    "filters lower filters.")
+class HintsFilter(object):
+
+    priority = 0
     
     def filter_location (self, location):
         """Returns a list of dictionaries of hints of a given location.
@@ -410,20 +482,61 @@ class HintsFilter (object):
         For example if your filter parses a directory it should return a list
         of hints of each encountered file.
         """
-        raise NotImplementedError
+        raise NotImplementedError()
     
     def __cmp__ (self, value):
-        assert isinstance (value, HintsFilter)
+        if not hasattr(value, 'priority'):
+            raise TypeError('%r has no \'priority\' attribute' % value)
+
         return self.priority - value.priority
 
-def normalize_hints(hints):
-    hints["location"] = urlutil.normalize(hints["location"])
+
+class MetadataRetriever(object):
+    """Manages objects that retrieve metadata from a location."""
+    def __init__(self):
+        self.filters = []
+
+    def filter_location(self, location):
+        """
+        Checks if a given location is a composite of 'hint' instances.
+        
+        throws UnsupportedLocationError"""
+        buggy_filters = []
+        
+        for loc_filter in self.filters:
+            try:
+                metadata = loc_filter.filter_location(location)
+                if metadata is None:
+                    logger.info('Metadata retriever %r is not returning an appropriate value, unregistring it.' % loc_filter)
+                    buggy_filters.append(loc_filter)
+                    continue
+                self._remove_filters(buggy_filters)
+                return metadata
+            except UnsupportedLocationError:
+                # Try next location filter
+                pass
+        self._remove_filters(buggy_filters)
+        raise UnsupportedLocationError()
+
+    def _remove_filters(self, filters):
+        for a_filter in filters:
+            self.filters.remove(a_filter)
+            
+    def add(self, location_filter):
+        self.filters.append(location_filter)
+        # Sort filters priority
+        self.filters.sort()
+    
+    def remove(self, location_filter):
+        self.filters.remove(location_filter)
+
 
 class MusicListGateway:
-    """This class wraps the MusicList interface in a friendlier one with a
-    method `add_files` easier to use then the `insert` method which expects
-    a hints `dict`. It also serves as a hints filter which is a list of client
-    objects which must provide the `filter_location` method.
+    """This class wraps the MusicList interface in a friendlier one
+    with a method `add_files` easier to use then the `insert` method
+    which expects a hints `dict`. It also serves as a hints filter
+    which is a list of client objects which must provide the
+    `filter_location` method.
     """
 
     class Handler:
@@ -441,25 +554,21 @@ class MusicListGateway:
     
     def __init__ (self, app):
         # Filters
-        self.__filters = []
+        self._metadata = MetadataRetriever()
         self._app = weakref.ref(app)
     
     music_list = None
     
-    def __filter_location (self, location):
-        for loc_filter in self.__filters:
-            hints = loc_filter.filter_location (location)
-            if hints is not None:
-                return hints
-        return None
+    def add_hints_filter(self, location_filter):
+        self._metadata.add(location_filter)
     
-    def add_files (self, filenames):
-        to_hint = lambda filename: {"location": urlutil.normalize(filename)}
-        return self.add_hints(map(to_hint, filenames))
+    def remove_hints_filter(self, location_filter):
+        self._metadata.remove(location_filter)
 
-    def add_hints (self, hints_list, insert = None):
+
+    def add_hints (self, hints_list, insert=None, known=()):
         assert insert is None or isinstance (insert, IntType)
-
+        logger.info('adding %r in %r' % (hints_list, insert))
         queue = OperationsQueue()
         queue.abort_on_failure = False
         
@@ -467,39 +576,47 @@ class MusicListGateway:
         
         handler.prepare_queue (self, queue)
         
-        i = 0
-        for h in hints_list:
-            pls = self.__filter_location (h["location"])
-            
-            if pls is not None and len (pls) > 0:
-                # normalize the returning elements
-                map(normalize_hints, pls)
-                # We add this to the queue so it is
-                # processed before the next file on the list
-                queue.append (self.add_hints(pls, insert))
+        for i, hint in enumerate(hints_list):
+            # prepare the operation 'AddFile'
+            try:
+                # try to check if the location is a composite of hints
+                pls = self._metadata.filter_location(hint["location"])
+
+                # Filter known files
+                pls = [p for p in pls if p not in known]
+                known += tuple(pls)
+                
+                for child_hint in pls:
+                    # normalize the location of the returning elements
+                    child_hint["location"] = urlutil.normalize(child_hint["location"])
+                                
+                queue.append(self.add_hints(pls, insert, known))
+
+                # composite hints have no children, thus do not need
+                # to be probed
                 continue
                 
+            except UnsupportedLocationError:
+                pass
+            
             ins = insert
-            if insert != None:
+            if ins is not None:
                 ins += i
             
-            a = AddFile (self.music_list, h, ins, self._app())
-            handler.prepare_add_file (self, a)
+            oper = AddFile(self.music_list, hint, ins, self._app())
+            handler.prepare_add_file(self, oper)
+
+            # Enqueue 'AddFile'
+            queue.append(oper)
             
-            queue.append (a)
-            
-            i += 1
+
         
         handler.finish_queue (self, queue)
         return queue
 
-    def add_hints_filter (self, location_filter):
-        self.__filters.append (location_filter)
-        # Sort filters priority
-        self.__filters.sort ()
-    
-    def remove_hints_filter (self, location_filter):
-        self.__filters.remove (location_filter)
+    def add_files(self, filenames):
+        return self.add_hints({"location": urlutil.normalize(filename)}
+                              for filename in filenames)
 
 
 class AudioMastering (gtk.VBox, operations.Listenable):
@@ -534,7 +651,7 @@ class AudioMastering (gtk.VBox, operations.Listenable):
                 del self.trapper
             
             def prepare_add_file (self, gateway, add_file):
-                add_file.listeners.append (self.trapper)
+                add_file.add_listener(self.trapper)
         
     
     disc_sizes = [21 * 60, 74 * 60, 80 * 60, 90 * 60]
@@ -549,12 +666,14 @@ class AudioMastering (gtk.VBox, operations.Listenable):
     def __init__ (self, application):
         gtk.VBox.__init__ (self)
         self._application = weakref.ref(application)
+        self._use_gap = application.preferences.connect("use_gap",
+                                                        self.on_use_gap)
         
         operations.Listenable.__init__ (self)
         self.__disc_size = 74 * 60
         self.update = True
         self.source = GtkMusicList ()
-        self.source.listeners.append (AudioMasteringMusicListener(self))
+        self.source.add_listener(AudioMasteringMusicListener(self))
         
         self.__gateway = AudioMastering.MusicListGateway (self)
         
@@ -570,6 +689,9 @@ class AudioMastering (gtk.VBox, operations.Listenable):
         self.keys = {
             "Delete": self.remove_selected,
         }
+
+    def on_use_gap(self, signal, use_gap):
+        self.update_disc_usage()
     
     def on_key_pressed(self, widget, evt):
         try:
@@ -582,10 +704,7 @@ class AudioMastering (gtk.VBox, operations.Listenable):
         self.__disc_size = size
         self.__size_list.set_active(AudioMastering.disc_sizes.index(size))
         self.update_disc_usage()
-        e = operations.Event (self)
-        for l in self.listeners:
-            if hasattr (l, "on_disc_size_changed"):
-                l.on_disc_size_changed (e)
+        self._notify('on_disc_size_changed', operations.Event(self))
         
     music_list_gateway = property (lambda self: self.__gateway)
     music_list = property (lambda self: self.source)
@@ -625,7 +744,7 @@ class AudioMastering (gtk.VBox, operations.Listenable):
         r.set_property ('editable', True)
         r.connect ('edited', self.__on_title_edited)
         lst.append_column (col)
-        col = gtk.TreeViewColumn ("Title", r, text = self.source.model.index_of("title"))
+        col = gtk.TreeViewColumn (_("Title"), r, text = self.source.model.index_of("title"))
         lst.append_column (col)
         
         r = gtk.CellRendererText()
@@ -657,7 +776,8 @@ class AudioMastering (gtk.VBox, operations.Listenable):
         lst.connect ("drag_data_get", self.__on_dnd_send)
         
     
-    def __generate_track (self, col, renderer, tree_model, treeiter, user_data = None):
+    def __generate_track(self, col, renderer, tree_model, treeiter,
+                         user_data=None):
         index = tree_model.get_path(treeiter)[0]
         renderer.set_property ('text', index + 1)
     
@@ -746,28 +866,24 @@ class AudioMastering (gtk.VBox, operations.Listenable):
                 
             hig_duration += hig_secs
         return hig_duration
-    # TODO: implement get_media_duration
 
     def get_preferences(self):
         return self._application().preferences
-        
-    def get_media_duration(self):
-        total = self.source.total_duration
-        num_tracks = len(self.source)
-        
-        if num_tracks > 0 and self.get_preferences().useGap:
-            total += (num_tracks - 1) * 2
-        
-        return total 
-    
+
+    preferences = property(get_preferences)
+
     def update_disc_usage (self):
+        
         if not self.update:
             return
+
+        self.source.use_gap = self.preferences.useGap
+        
         if self.source.total_duration > self.disc_size:
-            self.__capacity_exceeded.show ()
+            self.__capacity_exceeded.show()
             
         else:
-            self.__capacity_exceeded.hide ()
+            self.__capacity_exceeded.hide()
 
         # Flush events so progressbar redrawing gets done
         while gtk.events_pending():
@@ -785,18 +901,16 @@ class AudioMastering (gtk.VBox, operations.Listenable):
         self.__usage_label.set_text (dur)
             
         e = operations.Event(self)
-        for l in self.listeners:
-            l.on_contents_changed (e)
+        self._notify('on_contents_changed', e)
     
-    def __selection_changed (self, treeselection):
-        e = operations.Event (self)
-        for l in self.listeners:
-            l.on_selection_changed (e)
+    def __selection_changed(self, treeselection):
+        e = operations.Event(self)
+        self._notify('on_selection_changed', e)
 
     
     def get_selected (self):
         """Returns the selected indexes"""
-        store, path_list = self.__selection.get_selected_rows ()
+        store, path_list = self.__selection.get_selected_rows()
         
         if path_list is None:
             return []
