@@ -20,10 +20,11 @@
 # Authors: Tiago Cogumbreiro <cogumbreiro@users.sf.net>
 
 """Loads directory filters."""
-from os import path
 import glob
 
+from os import path
 from serpentine.mastering import HintsFilter
+from serpentine.errors import UnsupportedLocationError
 from serpentine import urlutil
 
 class DirectoryFilter (HintsFilter):
@@ -31,13 +32,13 @@ class DirectoryFilter (HintsFilter):
         url = urlutil.UrlParse(location)
 
         if not url.is_local or not path.isdir(url.path):
-            return
+            raise UnsupportedLocationError()
             
         files = glob.glob(path.join(url.path, "*"))
         files.sort()
 
         return [{"location": urlutil.normalize(loc)} for loc in files]
-
+    
 
 def create_plugin(app):
     app.add_hints_filter(DirectoryFilter())
