@@ -23,64 +23,59 @@ import zipfile, gnomevfs
 
 from xml.dom import minidom
 from xml.xpath import Evaluate
-from serpentine.mastering import HintsFilter
-from serpentine import urlutil
 from xml.parsers.expat import ExpatError
 
-import traceback
+from serpentine.mastering import HintsFilter
+from serpentine import urlutil
+from serpentine.errors import UnsupportedLocationError
 
-def safe_method (func):
-    def wrapper (self, *args, **kwargs):
-        try:
-            return func (self, *args, **kwargs)
-        except:
-            traceback.print_exc ()
+## import traceback
+
+## def safe_method (func):
+##     def wrapper (self, *args, **kwargs):
+##         try:
+##             return func (self, *args, **kwargs)
+##         except Exception:
+##             traceback.print_exc ()
     
-    return wrapper
+##     return wrapper
 
-class K3BFilter (HintsFilter):
-    def filter_location (self, location):
-        url = urlutil.UrlParse (location)
-        if not url.is_local:
-            fd = gnomevfs.open (url.unparse ())
-            def read_all ():
+class K3BFilter(HintsFilter):
+    def filter_location(self, location):
+        url = urlutil.UrlParse(location)
+
+        # use native approach
+        if url.is_local:
+            try:
+                fd = open(url.path)
+            except IOError:
+                # Could not open the filename
+                raise UnsupportedLocationError()
+            
+        # use gnomevfs
+        else:
+            fd = gnomevfs.open(url.unparse())
+            def read_all():
                 buff = ""
                 try:
                     while 1:
-                        buff += fd.read (1024)
+                        buff += fd.read(1024)
                 except gnomevfs.EOFError:
                     pass
                 return buff
             fd.read = read_all
-        else:
-            try:
-                fd = open (url.path)
-            except IOError:
-                # Could not open the filename
-                return
+
+            
 
         try:
-            zfile = zipfile.ZipFile (fd)
-        except zipfile.BadZipfile:
-            return
-        except IOError:
-            # it's not a file
-            return
-        
-        try:
-            buff = zfile.read ("maindata.xml")
-        except KeyError:
-            # zip file does not contain the file
-            fd.close ()
-            return
-        
-        fd.close ()
-        
-        try:
-            root = minidom.parseString (buff)
-        except ExpatError:
-            # Malformed xml
-            return
+            zfile = zipfile.ZipFile(fd)
+            buff = zfile.read("maindata.xml")
+            root = minidom.parseString(buff)
+            
+        except (zipfile.BadZipfile, IOError, KeyError, ExpatError):
+            raise UnsupportedLocationError()
+        finally:
+            fd.close()
         
         # Iterate over tracks
         hints_list = []
@@ -88,13 +83,14 @@ class K3BFilter (HintsFilter):
             try:
                 hints_list.append ({"location": node.attributes["url"].value})
             except KeyError:
-                # skip elements with not 'url' attribute set
+                # skip elements without the 'url' attribute set
                 pass
         
         # New versions of K3B changed the internal structure
         # since there's no version present for the file format we'll just
         # try both
-        for node in Evaluate ("/k3b_audio_project/contents/track/sources/file", root):
+        for node in Evaluate ("/k3b_audio_project/contents/track/sources/file",
+                              root):
             try:
                 hints_list.append ({"location": node.attributes["url"].value})
             except KeyError:
@@ -103,10 +99,10 @@ class K3BFilter (HintsFilter):
 
         return hints_list
     
-    filter_location = safe_method (filter_location)
+##    filter_location = safe_method(filter_location)
 
 def create_plugin (app):
     app.add_hints_filter (K3BFilter ())
     app.register_playlist_file_pattern ("K3B Audio Project", "*.k3b")
     app.register_music_file_pattern ("K3B Audio Project", "*.k3b")
-    
\ No newline at end of file
+    
