@@ -21,24 +21,22 @@
 
 from serpentine import operations, xspf
 from serpentine.common import SafeFileWrite
-from xml.dom import minidom
 import traceback
 
-class SavePlaylist (operations.Operation):
-    def __init__ (self, music_list, filename):
-        super (SavePlaylist, self).__init__ ()
-        
+class SavePlaylist(operations.Operation):
+    def __init__(self, music_list, filename):
+        super(SavePlaylist, self).__init__()
         self.music_list = music_list
         self.filename = filename
 
-    def start (self):
+    def start(self):
         status = operations.SUCCESSFUL
-        fp = SafeFileWrite (self.filename)
+        fp = SafeFileWrite(self.filename)
         try:
             self._save (fp)
             fp.close ()
             
-        except:
+        except Exception:
             traceback.print_exc()
             status = operations.ERROR
             # Abort changes
@@ -47,9 +45,12 @@ class SavePlaylist (operations.Operation):
         
         self._send_finished_event (status)
     
-    def _save (fp):
+    def _save(self, fp):
         raise NotImplementedError
 
+    def get_is_running(self):
+        return false
+
 class SavePLS (SavePlaylist):
     def _save (self, fp):
         fp.write ("[playlist]\n")
@@ -96,4 +97,4 @@ def create_plugin (app):
     # Register factories
     app.savePlaylist.register (factory=SaveM3U,  extension = ".m3u",  description = "MP3 Playlist File")
     app.savePlaylist.register (factory=SavePLS,  extension = ".pls",  description = "PLS Audio Playlist")
-    app.savePlaylist.register (factory=SaveXSPF, extension = ".xspf", description = "XML Shareable Playlist Format")
\ No newline at end of file
+    app.savePlaylist.register (factory=SaveXSPF, extension = ".xspf", description = "XML Shareable Playlist Format")
