@@ -22,6 +22,7 @@ from os import path
 
 from serpentine import urlutil
 from serpentine.mastering import HintsFilter
+from serpentine.errors import UnsupportedLocationError
 from serpentine import xspf
 
 class XspfFilter(HintsFilter):
@@ -29,14 +30,14 @@ class XspfFilter(HintsFilter):
         url = urlutil.UrlParse(location)
         
         if not url.is_local:
-            return
+            raise UnsupportedLocationError()
 
         hints_list = []
         p = xspf.Playlist()
         try:
-            p.parse (url.path)
+            p.parse(url.path)
         except Exception:
-            return
+            raise UnsupportedLocationError()
             
         basename = path.basename(url.path)
         
