@@ -27,14 +27,14 @@ from xml.dom import minidom
 from xml.xpath import Evaluate
 from xml.dom.minidom import getDOMImplementation
 
-class _Field (object):
-	def __init__ (self, id = None, convert = None):
+class _Field(object):
+	def __init__(self, id=None, convert=None):
 		self.id = id
 		self.convert = convert
 		
 	data = None
 		
-	def toxml (self, doc, root):
+	def toxml(self, doc, root):
 		if self.data is not None:
 			node = doc.createElement (self.id)
 			node.appendChild (doc.createTextNode (str (self.data)))
@@ -125,21 +125,6 @@ class Playlist (_Struct):
 		# Iterate over tracks
 		for track_node in Evaluate ("/playlist/trackList/track", root):
 			t = Track()
-			# Get each field
-			#for field in t._fields:
-			#	try:
-			#		val = Evaluate ("string(%s)" % (field), track_node).strip()
-			#		convert = t._fields[field].convert
-			#		if convert:
-			#			val = convert (val)
-			#		setattr (t, field, val)
-			#	except ValueError:
-			#		pass
 			t._parse_node (track_node)
 			self.tracks.append(t)
 
-if __name__ == '__main__':
-	import sys
-	p = Playlist ()
-	for arg in sys.argv[1:]:
-		print to_xml_string (p.parse_filename (arg))
